import math
import pygame
import pyui
from hoop import engine

from sprites import Bullet

world = 'none'

"""
Keeps track of which keys are presently pressed for a specific soldier.
"""
class Controller:

    def __init__(self, soldier):
        self.soldier = soldier
        self.up = 0
        self.down = 0
        self.left = 0
        self.right = 0
        self.fire = 0
        self.throw = 0
    """
    All possible orders are set by this update
    """
    def update(self):
        if not self.soldier.alive:
            return
        if self.up:
            self.soldier.jump()
        #self.down not presently used
        if self.left:
            self.soldier.moveLeft()
        if self.right:
            self.soldier.moveRight()
        if self.fire:
            self.soldier.fire()
        if self.throw:
            self.soldier.throw()

    """
    Updates keboard input to current use
    """
    def processKeys(self):
        pressed = pygame.key.get_pressed()
        self.up = pressed[pyui.locals.K_UP]
        self.down = pressed[pyui.locals.K_DOWN]
        self.left = pressed[pyui.locals.K_LEFT]
        self.right = pressed[pyui.locals.K_RIGHT]


    """
    Update all mouse input to current settings and calculates the angle
    """
    def processMouse(self):
        #Read mouse cursor position on screen
        (mxs, mys) = pygame.mouse.get_pos()
        (button1, button2, button3) = pygame.mouse.get_pressed()
        #Convert to world coordinates
        (mx, my) = engine.screenToWorld(mxs, mys)
        dx = mx - self.soldier.posX
        dy = my - self.soldier.posY
        angle = 0
        if dx > 0:
            angle = math.atan(dy/dx)
        elif dx < 0:
            angle = math.pi + math.atan(dy/dx)
        elif dx == 0:
            if dy > 0:
                angle = math.pi/2
            else:
                angle = -math.pi/2

        self.soldier.gun.facing = angle*180/math.pi

        self.fire = button1
        self.throw = button3
        
        """
        Used by server to poll for mouse and keyboard input 
        """
    def processPeripheralsServer(self):
        self.processKeys()
        self.processMouse()

        """
        Used by client to poll for mouse input 
        """
    def processPeripheralsClient(self):
        self.processMouse()
